﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace JournyxAPISample
{
    partial class Program
    {
        public void demoApprovalTemplates()
        {
            string thedate = "20110620";

            // Create an approval plan.  Note that since these are not core database
            // types, you don't need to get a default record first.
            journyx.ApprovalObject approval = new journyx.ApprovalObject {
                name = "Test Approval Plan",
                type = "time",
                bizid = "default2",
                userbefore = 1,
                userafter = 2,
                approverafter = 3,
                backupafter = 4,
                userrejected = 1,
                userapproved = 1,
                approversubmitted = 1,
                approver_1 = USER,
                approver_2 = USER,
                backup_approver_1 = testuser_id
            };

            api.CreateApprovalPlan(session_key, approval);

            // Now retrieve them.  Pass an empty ID list to get them all.
            journyx.ApprovalObject[] approvals = api.getApprovalPlans(session_key, new string[] {});
            foreach (journyx.ApprovalObject appr in approvals) {
                Console.WriteLine("... approval plan {0}, bizid {1}", appr.id, appr.bizid);
            }

            // Assign it to a user, and unassign.  It can be done by ID or name.
            api.AssignApprovalPlan(session_key, approvals[0].id, new string[] { testuser_id });

            // To demonstrate the submission and approval process, we actually need to be
            // logged in as two different users.  So let's log in as the test user.
            string session2_key;
            try {
                session2_key = api.login(testuser_id, "test", 1);
            }
            catch (System.ServiceModel.FaultException) {
                api.changeUserPassword(testuser_id, testuser_id, "test");
                session2_key = api.login(testuser_id, "test", 1);
            }

            // As the second user: Begin by accessing (thus implicitly creating) the sheet for our test date.
            string sheet_id = api.getSheetIDByDate(session2_key, "time", thedate);

            // As the second user: It doesn't have any hours, but let's submit it anyway.
            api.submitSheet(session2_key, "time", sheet_id);
            
            // As the main (admin) user: reject the sheet.
            journyx.SheetStatusRecord newstatus = new journyx.SheetStatusRecord {
                id = sheet_id,
                change = "reject",
                reason = "no reason"
            };
            journyx.SheetStatusRecord[] sheetresults = api.ChangeSheetStatus(session_key, new journyx.SheetStatusRecord[] { newstatus });
            foreach (journyx.SheetStatusRecord status in sheetresults) {
                Console.WriteLine("... sheet status {0}, {1}, {2}", status.id, status.status, status.reason);
            }

            // Now unassign the approval plan.
            api.UnassignApprovalPlan(session_key, approvals[0].id, new string[] { testuser_id });
        }
    }
}