﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace JournyxAPISample
{
    partial class Program
    {
        public void demoAttributes()
        {
            // Start off by looking up an attribute type we know is in the default install,
            // the "budget" attribute of projects.
            string attr_id = api.getAttributeTypeByName(session_key, "projects", "Budget");

            // Rename it to something else.
            api.modifyAttributeTypeName(session_key, attr_id, "Planned Expenditures");
            api.modifyAttributeTypeDescription(session_key, attr_id, "How much are we going to spend?");

            // Now get the full attribute record.
            journyx.AttributeTypeRecord attr_rec = api.getAttributeTypeRecordByName(session_key, "projects", "Budget");
            Console.WriteLine("... attribute {0} {1}", attr_rec.pname, attr_rec.attr_type);

            // And rename it back.
            api.modifyAttributeTypeName(session_key, attr_id, "Budget");

            // Now let's add a new one.  First, an API function to check if a data type is valid.
            // We'll use the root project to see if it works.

            // checkAttributeDataType returns nonzero if the specified string is a valid attribute type
            Debug.Assert(api.checkAttributeDataType(session_key, "STRING") != 0);
            attr_id = api.addAttributeType(session_key, "projects", "jxapi demo 1", "STRING");
            Console.WriteLine("... new attribute ID is {0}", attr_id);
            // You can set a default value which is used whenever the attribute is not explicitly set.
            api.setAttributeTypeDefaultValue(session_key, attr_id, "default value");
            Console.WriteLine("... default value is '{0}'", api.getAttributeTypeDefaultValue(session_key, attr_id));
            // You can remove the default value.
            api.removeAttributeTypeDefaultValue(session_key, attr_id);
            // checkAttributeValue returns nonzero if the specified value matches the given attribute type
            Debug.Assert(api.checkAttributeValue(session_key, "STRING", "test value") != 0);
            api.setAttribute(session_key, "projects", "root", attr_id, "test value", true);
            string attr_value = api.getAttribute(session_key, "projects", "root", attr_id);
            Console.WriteLine("... new attribute value is {0}", attr_value);
            // You can remove an attribute value from an object without deleting the attribute itself.
            api.deleteAttribute(session_key, "projects", "root", attr_id);
            // Clean up.
            api.deleteAttributeType(session_key, attr_id);
            
            // Because of limitations on overloading in the jxAPI, there are parallel methods for
            // getAttribute, setAttribute, checkAttributeValue, and {get,set}AttributeTypeDefaultValue
            // for numeric and integer values.
            Debug.Assert(api.checkAttributeDataType(session_key, "INTEGER") != 0);
            attr_id = api.addAttributeType(session_key, "projects", "jxapi demo 2", "INTEGER");
            Console.WriteLine("... new integer attribute ID is {0}", attr_id);
            api.setAttributeTypeDefaultValueInteger(session_key, attr_id, 42);
            Console.WriteLine("... default value is '{0}'", api.getAttributeTypeDefaultValueInteger(session_key, attr_id));
            Debug.Assert(api.checkAttributeValueInteger(session_key, "INTEGER", 2) != 0);
            api.setAttributeInteger(session_key, "projects", "root", attr_id, 2, true);
            int int_attr_value = api.getAttributeInteger(session_key, "projects", "root", attr_id);
            Console.WriteLine("... new integer attribute value is {0}", int_attr_value);
            api.deleteAttributeType(session_key, attr_id);

            Debug.Assert(api.checkAttributeDataType(session_key, "NUMBER") != 0);
            attr_id = api.addAttributeType(session_key, "projects", "jxapi demo 3", "NUMBER");
            Console.WriteLine("... new float attribute ID is {0}", attr_id);
            api.setAttributeTypeDefaultValueNumber(session_key, attr_id, 42.7);
            Console.WriteLine("... default value is '{0}'", api.getAttributeTypeDefaultValueNumber(session_key, attr_id));
            Debug.Assert(api.checkAttributeValueNumber(session_key, "NUMBER", 2.1) != 0);
            api.setAttributeNumber(session_key, "projects", "root", attr_id, 2.1, true);
            double num_attr_value = api.getAttributeNumber(session_key, "projects", "root", attr_id);
            Console.WriteLine("... new float attribute value is {0}", num_attr_value);
            api.deleteAttributeType(session_key, attr_id);

            // Now let's add an attribute again, this time with selection values.
            // Add it as a full record, to show use of that function.
            attr_rec = new journyx.AttributeTypeRecord {
                attr_type = "ENUM_STRING_20",
                description = "jxapi demo attribute",
                pname = "jxapi demo attribute",
                table_name = "projects"
            };
            attr_id = api.addFullAttributeType(session_key, attr_rec);

            // Add some selection values.  Note that for integer and numeric values there are
            // parallel functions addAttributeTypeSelectionValueNumber and addAttributeTypeSelectionValueInteger
            // (not shown here, but identical to addAttributeTypeSelectionValue except for the
            // type of the value argument).
            string[] selections = { "Test1", "Test2", "Test Three", "Four", "555555" };
            foreach (string sel in selections) {
                api.addAttributeTypeSelectionValue(session_key, attr_id, sel, false);
            }

            string[] values = api.getAttributeTypeSelectionValues(session_key, attr_id);
            Console.WriteLine("... selection values: {0}", formatList(values));

            // Add another value and make it the default using its id.
            string value_id = api.addAttributeTypeSelectionValue(session_key, attr_id, "foo", false);
            api.setAttributeTypeSelectionDefault(session_key, value_id);

            // Now let's see if the default value is reflected in a newly added project.
            journyx.ProjectRecord prec = api.getDefaultProject(session_key);
            prec.parent = "root";
            prec.name = "test project";
            prec.id = "testproject00";
            prec.creator = USER;
            string proj_id = api.addFullProject(session_key, prec);

            Console.WriteLine("... attribute for test project is {0}", api.getAttribute(session_key, "projects", "testproject00", attr_id));

            // You can delete an individual selection value like this:
            api.deleteAttributeTypeSelectionValue(session_key, value_id);

            // And clean up.
            api.removeProject(session_key, proj_id);
            api.deleteAttributeType(session_key, attr_id);
        }
    }
}