﻿//
// Journyx jxAPI Sample for C#.NET 3.5
//
// This sample exercises many parts of the jxAPI, and is intended both as a reference
// for client developers and as a way to test a Timesheet installation for working API
// functionality.
//
// To run this sample as-is, configure the variables near the top of the program to
// specify the URL of your server, the name of the installing user, and a test password.
// Do not run this sample against a production server; use a temporary Timesheet install
// instead.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace JournyxAPISample
{
    partial class Program
    {
        // Configure these variables as appropriate for your server.
        //   JX_URL - the SOAP endpoint URL of your server.
        const string JX_URL = "http://vulcan:50104/jtcgi/jxapi_wsi.pyc";
        //   USER - the name of the installing user
        const string USER = "rob";
        //   PASSWORD - the password of the installing user 
        const string PASSWORD = "test";

        public journyx.jxapi api;         // WCF proxy for the jxAPI
        public string        session_key; // jxAPI login session key

        public string testgroup_id;    // id of a test group used in various examples
        public string testuser_id;     // id of a test (non-admin) user

        //
        // Log into the jxAPI as the specified user.  If a fault occurs on
        // the first attempt to log in, assume that it is because the server
        // is freshly installed, and the installing user needs its password
        // changed.
        //
        public void Login()
        {
            try {
                session_key = api.login(USER, PASSWORD, 1);
            }
            catch (System.ServiceModel.FaultException) {
                api.changeUserPassword(USER, USER, PASSWORD);
                session_key = api.login(USER, PASSWORD, 1);
            }
        }

        public void PrintServerInformation()
        {
            int[] apiVersion;

            Console.WriteLine("Timesheet version: {0}", api.version());
            apiVersion = api.apiVersion();
            Console.WriteLine("jxAPI version:     {0}.{1}.{2}.{3}", apiVersion[0],apiVersion[1],apiVersion[2],apiVersion[3]);
            Console.WriteLine();

            Console.WriteLine("Server OS:         {0}", api.uname(session_key));
            Console.WriteLine("Host name:         {0}", api.hostname(session_key));
            Console.WriteLine("Server Admin:      {0} <{1}> ({2})", api.adminName(session_key), api.adminEmail(session_key), api.adminTelephone(session_key));
            Console.WriteLine();

            Console.WriteLine("Install date:      {0}", api.installDate(session_key));
            Console.WriteLine("Licensed users:    {0}", api.licensedUsers(session_key));
            Console.WriteLine("Licensed host:     {0}", api.licensedHost(session_key));
            Console.WriteLine("Expiration date:   {0}", api.expirationDate(session_key));
            Console.WriteLine("Company name:      {0}", api.companyName(session_key));
            Console.WriteLine("Customer number:   {0}", api.customerNumber(session_key));
            Console.WriteLine();
        }

        //
        // Utility functions used by the rest of this demo.
        //
        public string jxTodaysDate()
        {
            return DateTime.Today.ToString("yyyyMMdd");
        }

        public string formatList(string[] strs)
        {
            return String.Join(", ", strs);
        }

        public void createTestData()
        {
            journyx.GroupRecord grec;

            grec = api.getGroup(session_key, new journyx.GroupRecord { id = "testgrp" });
            if (String.IsNullOrEmpty(grec.id)) {
                grec = api.getDefaultGroup(session_key);
                grec.name = "TestGroup";
                grec.id = "testgrp";
                grec.description = "Test Group";
                testgroup_id = api.addFullGroup(session_key, grec);
            } else {
                testgroup_id = grec.id;
            }

            journyx.UserRecord urec;
            urec = api.getUser(session_key, new journyx.UserRecord { id = "testuser0" });
            if (String.IsNullOrEmpty(urec.id)) {
                urec = api.getDefaultUser(session_key);
                urec.full_name = "Tom Tester";
                urec.user_login = "testuser0";
                urec.id = "testuser0";
                urec.time_gui = "Standard Time Entry Grid";
                urec.expense_gui = "Standard Expense Entry Grid";
                urec.mileage_gui = "Standard Mileage Entry Grid";
                testuser_id = api.addFullUser(session_key, urec);
            } else {
                testuser_id = urec.id;
            }

            api.addUserToGroup(session_key, testuser_id, testgroup_id);
            api.addUserToGroup(session_key, USER, testgroup_id);
            api.addCodeToGroup(session_key, "Coding", testgroup_id);
            api.addCodeToGroup(session_key, "Dev", testgroup_id);
            api.addSubcodeToGroup(session_key, "Unbillable", testgroup_id);
        }

        public void destroyTestData()
        {
            api.removeSubcodeFromGroup(session_key, "Unbillable", testgroup_id);
            api.removeCodeFromGroup(session_key, "Dev", testgroup_id);
            api.removeCodeFromGroup(session_key, "Coding", testgroup_id);
            api.removeUserFromGroup(session_key, USER, testgroup_id);
            api.removeUserFromGroup(session_key, testuser_id, testgroup_id);
            api.removeGroup(session_key, testgroup_id);
            api.removeUser(session_key, testuser_id);
        }

        public void Run()
        {
            // The Journyx server does not support the Expect: 100-continue header,
            // so you must disable it before making any calls to the API.
            System.Net.ServicePointManager.Expect100Continue = false;

            api = new journyx.jxapiClient("jxapi", JX_URL);

            Login();
            PrintServerInformation();
            createTestData();
            demoJXCoreTypes();
            // Note: the Approval Template demo is commented out here because, since there
            // is currently no way to delete approval templates from the jxAPI, it does not
            // return the database to the same state it was in before running.
            // demoApprovalTemplates();
            demoAttributes();
            demoMemorizedEntries();
            demoJournals();
            demoAccruals();
            demoRateRules();
            destroyTestData();
            api.logout(session_key);
        }

        static void Main(string[] args)
        {
            Program prog = new Program();
            prog.Run();

            Console.WriteLine("Press any key...");
            ConsoleKeyInfo cki = Console.ReadKey();
        }
    }
}
