﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace JournyxAPISample
{
    partial class Program
    {
        public void demoRateRules()
        {
            // First, create and modify an individual rate rule.

            // This example rule matches hours > 40 in any project, code, subcode, or subsubcode.
            journyx.RateRule rule = new journyx.RateRule {
                name = "testrule0",
                description = "test rule 0",
                project = new string[] { "wildProj" },
                code = new string[] { "wildCode" },
                subcode = new string[] { "wildSub" },
                othercode = new string[] { "wildOther" },
                hours = 40.0,
                timeframe = "Week",
                condition = "g"
            };

            string rule_id = api.createRateRule(session_key, rule);
            Console.WriteLine("rule created: {0}", rule_id);

            // Now read it and modify it.
            rule = api.getRateRule(session_key, "testrule0");
            rule.description = "MODIFIED";
            string rule_id2 = api.modifyRateRule(session_key, "testrule0", rule);
            Debug.Assert(rule_id == rule_id2);

            // Return the names of all rules.
            string[] rulenames = api.getRateRuleNames(session_key);
            foreach (string r in rulenames) {
                Console.WriteLine("... rule: {0}", r);
            }

            // Now create a rule set encompassing the rule we already created.
            // In this example, the rule matches hours > 40/week, so we'll use the rule
            // set to pay time and half while leaving the billed rate the same.
            journyx.RuleSet ruleset = new journyx.RuleSet {
                name = "testruleset0",
                description = "test rule set 0",
                payRateOrigin = "User",
                billRateOrigin = "-none-",
                rules = new journyx.RuleSetRow[] {
                    new journyx.RuleSetRow { rule = "testrule0", paytype = "Multiply", payamount = 1.5 }
                }
            };
            string ruleset_id = api.createRuleSet(session_key, ruleset);

            // And we can modify it, read it, and get rule set names just like
            // we can with individual ruless.
            ruleset = api.getRuleSet(session_key, "testruleset0");
            ruleset.description = "MODIFIED";
            string ruleset_id2 = api.modifyRuleSet(session_key, "testruleset0", ruleset);
            Debug.Assert(ruleset_id == ruleset_id2);

            string[] rulesetnames = api.getRuleSetNames(session_key);
            foreach (string r in rulesetnames) {
                Console.WriteLine("... ruleset: {0}", r);
            }

            // Now that we have a rule set, assign it to a user.
            api.assignRuleSetToUser(session_key, testuser_id, "testruleset0");
            // ... or remove their rule set.
            api.assignRuleSetToUser(session_key, testuser_id, "");

            // And finally, delete what we created.
            api.deleteRuleSet(session_key, "testruleset0");
            api.deleteRateRule(session_key, "testrule0");
        }
    }
}
