
import jxapi.*;
import jxapi.JxapiStub.*;
import java.lang.reflect.*;

import org.apache.axis2.*;
import org.apache.axis2.transport.http.*;

// Axis2 uses checked exceptions, throwing a (subclass of) java.rmi.RemoteException
// on a SOAP fault or other error in the API stub.
import java.rmi.RemoteException;

import static java.lang.System.out;

public class jxapi_demo {
    /*
     * Set the following variables to point to your test server
     * and its installing user.
     */
    static String JXAPI_URL = "http://vulcan:50100/jtcgi/jxapi_wsi.pyc";
    static String USER = "rob";
    static String PASSWORD = "test";

    static JxapiStub api;
    static String    session_key;

    static String    testgroup_id;
    static String    testuser_id;

    /*
     * Log in to the Timesheet server.  The try/catch here shows how to
     * handle the case of an expired password.  (For this demo, we assume
     * that if the password is "expired" it means the server has just been
     * set up, so we simply change the user's password to a test value.)
     */
    public void login() throws RemoteException {
        try {
            session_key = api.login(USER, PASSWORD, 1);
        } catch (RemoteException e) {
            api.changeUserPassword(USER, USER, PASSWORD);
            session_key = api.login(USER, PASSWORD, 1);
        }
    }

    public void printServerInformation() throws RemoteException {
        out.format("Timesheet version: %s\n", api.version());
        int[] apiVersion = api.apiVersion().get_int();
        out.format("jxAPI version:     %s.%s.%s.%s\n\n", apiVersion[0], apiVersion[1], apiVersion[2], apiVersion[3]);
        out.format("Server OS:         %s\n", api.uname(session_key));
        out.format("Host name:         %s\n", api.hostname(session_key));
        out.format("Server admin:      %s <%s> (%s)\n", api.adminName(session_key), api.adminEmail(session_key),
                api.adminTelephone(session_key));
        out.format("Install date:      %s\n", api.installDate(session_key));
        out.format("Licensed users:    %s\n", api.licensedUsers(session_key));
        out.format("Licensed host:     %s\n", api.licensedHost(session_key));
        out.format("Expiration date:   %s\n", api.expirationDate(session_key));
        out.format("Company name:      %s\n", api.companyName(session_key));
        out.format("Customer number:   %s\n\n", api.customerNumber(session_key));
    }

    /*
     * Records created directly by their constructor have null string fields,
     * but passing a null field to the jxAPI will cause an exception on the
     * client side because the WSDL does not allow it.  This utility function
     * uses reflection to construct an empty record object where the string
     * fields contain empty strings instead of nulls, usable as a search
     * pattern for example.
     */
    public Object newRecord(Class cls) {
        try {
            Constructor ctor = cls.getConstructor(new Class[0]);
            Object rec = ctor.newInstance(new Object[0]);

            for (Method m : cls.getDeclaredMethods()) {
                if (!m.getName().startsWith("set")) {
                    continue;
                }

                Class partypes[] = m.getParameterTypes();
                if (partypes.length != 1 || partypes[0] != String.class) {
                    continue;
                }

                Object args[] = new Object[1];
                args[0] = "";
                m.invoke(rec, args);
            }

            return rec;
        } catch (Throwable e) {
            throw new RuntimeException(e);
        }
    }

    /*
     * It is very common to build a record which is entirely empty except for
     * the ID field, for use as a search pattern to retrieve a specific
     * record.  This utility function does that, so that it can be easily
     * inlined into a jxAPI call.
     */

    public Record newRecordWithID(Class<? extends Record> cls, String id) {
        try {
            Class partypes[] = new Class[1];
            partypes[0] = String.class;
            Method meth = cls.getMethod("setId",partypes);
            Object arglist[] = new Object[1];
            arglist[0] = id;
            Record rec = (Record)newRecord(cls);
            Object retobj = meth.invoke(rec, arglist);

            return rec;
        } catch (Throwable e) {
            throw new RuntimeException(e);
        }
    }

    /*
     * String convenience methods.
     */
    public String join(String[] strings, String sep) {
        if(strings.length == 0) return "";
        if(strings.length == 1) return strings[0];
        StringBuilder sb = new StringBuilder();
        sb.append(strings[0]);
        for(int i = 1; i < strings.length; i++) {
            sb.append(sep);
            sb.append(strings[i]);
        }
        return sb.toString();
    }
        
    public String formatList(String[] list) {
        return join(list, ", ");
    }

    public boolean isEmpty(String str) {
        return (str == null || str.length() == 0);
    }

    /*
     * The following two methods create and destroy a couple of test
     * data items used by other parts of the demo.  Please see the
     * applicable section of the demo for further detail on how the
     * API methods used here work.
     */
    void createTestData() throws RemoteException {
        GroupRecord grec = new GroupRecord();

        grec = api.getGroup(session_key, (GroupRecord)newRecordWithID(GroupRecord.class, "testgrp"));
        if (isEmpty(grec.getId())) {
            grec = api.getDefaultGroup(session_key);
            grec.setName("TestGroup");
            grec.setId("testgrp");
            grec.setDescription("Test Group");
            testgroup_id = api.addFullGroup(session_key, grec);
        } else {
            testgroup_id = grec.getId();
        }

        UserRecord urec = new UserRecord();
        urec = api.getUser(session_key, (UserRecord)newRecordWithID(UserRecord.class, "testuser0"));
        if (isEmpty(urec.getId())) {
            urec = api.getDefaultUser(session_key);
            urec.setFull_name("Tom Tester");
            urec.setUser_login("testuser0");
            urec.setId("testuser0");
            urec.setTime_gui("Standard Time Entry Grid");
            urec.setExpense_gui("Standard Expense Entry Grid");
            urec.setMileage_gui("Standard Mileage Entry Grid");
            testuser_id = api.addFullUser(session_key, urec);
        } else {
            testuser_id = urec.getId();
        }

        // Add some codes and users etc. that we will need later.
        api.addUserToGroup(session_key, testuser_id, testgroup_id);
        api.addUserToGroup(session_key, USER, testgroup_id);
        api.addCodeToGroup(session_key, "Coding", testgroup_id);
        api.addCodeToGroup(session_key, "Dev", testgroup_id);
        api.addSubsubcodeToGroup(session_key, "Unbillable", testgroup_id);
    }

    void destroyTestData() throws RemoteException {
        api.removeSubsubcodeFromGroup(session_key, "Unbillable", testgroup_id);
        api.removeCodeFromGroup(session_key, "Dev", testgroup_id);
        api.removeCodeFromGroup(session_key, "Coding", testgroup_id);
        api.removeUserFromGroup(session_key, USER, testgroup_id);
        api.removeUserFromGroup(session_key, testuser_id, testgroup_id);
        api.removeUser(session_key, testuser_id);
        api.removeGroup(session_key, testgroup_id);
    }

    public void run(String[] args) {
        try {
            api = new JxapiStub(JXAPI_URL);
            // The jxAPI does not support chunked HTTP.
            api._getServiceClient().getOptions().setProperty(HTTPConstants.CHUNKED, false);

            out.println("jxAPI Demo for Java SE 6 and Apache Axis 2");
            out.println("");
            login();
            out.format("Session key: %s\n\n", session_key);
            printServerInformation();
            createTestData();
            
            new jxapi_core_demo(this).run();
            new jxapi_advanced_demo(this).run();
            new jxapi_attribute_demo(this).run();

            destroyTestData();
            api.logout(session_key);
            out.println("Logged out.\n");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public static void main(String[] args) {
        jxapi_demo demo = new jxapi_demo();
        demo.run(args);
    }
}

