package com.journyx.jxapidemo;

import jxapi.*;
import com.journyx.jxapi.*;

import java.rmi.RemoteException;
import static java.lang.System.out;

public class jxapi_advanced_demo {
    static jxapi_demo app;
    static String session_key;
    static JxapiStub api;

    public jxapi_advanced_demo(jxapi_demo demoapp) {
        app = demoapp;
        session_key = jxapi_demo.session_key;
        api = jxapi_demo.api;
    }

    public void run () throws RemoteException {
        demoMemorizedEntries();
        demoAccruals();
        demoJournals();
        // demoRateRules();
        // Note: The approval plan demo does not run by default, because
        // it is impossible to return the database to its previous state
        // afterward.  Note that if you uncomment the following call,
        // the demo app will likely throw an exception when trying to
        // remove the test user.
        //
        // demoApprovalPlans();
    }

    public void demoMemorizedEntries() throws RemoteException {
        // A memorized entry holds a combination of group, project,
        // codes, and comment which is shown as a row on the user's
        // entry screen.  For this demo we'll create one using some
        // information that we already know is in the default database.

        MemorizedEntry mem_entry = (MemorizedEntry) app.newRecord(MemorizedEntry.class);
        mem_entry.setGroup("Any");
        mem_entry.setProject("root");
        mem_entry.setCode1("Coding");
        mem_entry.setCode2("Regular");
        mem_entry.setCode3("Unbillable");
        mem_entry.setComment("Test memorized entry");

        String mentry_id = api.createUserMemorizedEntry(session_key, "time", jxapi_demo.USER, mem_entry);
        out.format("... memorized entry id %s\n", mentry_id);
        
        mem_entry.setCode1("Dev");
        String mentry_id2 = api.modifyUserMemorizedEntry(session_key, "time", mentry_id, mem_entry);
        assert mentry_id.equals(mentry_id2);

        MemorizedEntry[] mem_entries = api.getUserMemorizedEntries(session_key, "time", jxapi_demo.USER);
        for (MemorizedEntry mem_entry1 : mem_entries) {
            out.format("memorized entry for %s: %s\n", jxapi_demo.USER, mem_entry1.getId());
        }

        api.deleteUserMemorizedEntry(session_key, "time", mentry_id);

        // Groups can have memorized entries too.
        MemorizedEntry g_mem_entry = new MemorizedEntry();
        g_mem_entry.setGroup(jxapi_demo.testgroup_id);
        g_mem_entry.setProject("root");
        g_mem_entry.setCode1("Coding");
        g_mem_entry.setCode2("Regular");
        g_mem_entry.setCode3("Unbillable");
        g_mem_entry.setComment("Test group entry");

        String g_mentry_id = api.createGroupMemorizedEntry(session_key, "time", jxapi_demo.testgroup_id, mem_entry);
        out.format("... group mem entry id %s\n", g_mentry_id);

        g_mem_entry.setCode1("Dev");
        String g_mentry_id2 = api.modifyGroupMemorizedEntry(session_key, "time", g_mentry_id, mem_entry);
        assert g_mentry_id.equals(g_mentry_id2);

        MemorizedEntry[] g_mem_entries = api.getGroupMemorizedEntries(session_key, "time", jxapi_demo.testgroup_id);
        for (MemorizedEntry g_mem_entry1 : g_mem_entries) {
            out.format("memorized entry for %s: %s\n", jxapi_demo.testgroup_id, g_mem_entry1.getId());
        }

        api.deleteGroupMemorizedEntry(session_key, "time", g_mentry_id);
    }

    public void demoAccruals() throws RemoteException {
        // Managing accruals is relatively simple; jxAPI methods to
        // create, modify, get, and delete them are provided.

        AccrualRecord acc_rec = new AccrualRecord();
        acc_rec.setId("");
        acc_rec.setCode("Holiday");
        acc_rec.setType("Soft");
        acc_rec.setHours(10.0);
        
        String acc_id = api.createUserAccrual(session_key, jxapi_demo.testuser_id, acc_rec);
        out.format("created accrual with id %s\n", acc_id);

        acc_rec.setHours(24.0);
        String acc_id2 = api.modifyUserAccrual(session_key, acc_id, acc_rec);
        assert acc_id.equals(acc_id2);

        AccrualRecord[] accruals = api.getUserAccruals(session_key, jxapi_demo.testuser_id);
        for (AccrualRecord accrual : accruals) {
            out.format("... accrual %s: %s %s", accrual.getId(), accrual.getCode(), accrual.getHours());
        }

        api.deleteUserAccrual(session_key, acc_id);
    }

    public void demoJournals() throws RemoteException {
        // Journals can be connected to projects or to specific entries.
        // A journal actually consists of multiple notes, the first of
        // which is automatically created at the same time as the
        // journal.
        //
        // For this demo we will create a test project to attach
        // journals to.
        ProjectRecord prec = (ProjectRecord)app.newRecordWithID(ProjectRecord.class, "jtest00");
        prec.setName("jtest00");
        prec.setParent("root");
        prec.setCreator(jxapi_demo.USER);
        String project_id = api.addFullProject(session_key, prec);

        // Journal names must be unique.  If you call
        // createJournalForProject with the same journal name as an
        // existing journal, it will return the id of that journal
        // instead of creating a new one.
        String jid = api.createJournalForProject(session_key, project_id, "testjournal0");
        out.format("journal created: %s\n", jid);

        // To get the journal IDs in a project:
        String[] journal_ids = api.getJournalsForProject(session_key, project_id);
        for (String journal_id : journal_ids) {
            out.format("... journal id in project: %s\n", journal_id);
        }

        // Add some notes.  The text can be any length, but we'll just
        // use short ones for this demo.
        api.createNote(session_key, jid, "test note one", "", "");
        api.createNote(session_key, jid, "test note two", "", "");

        // getNotesForJournal returns a list of note records.  The last
        // three parameters (userid, start date, end date) can be used
        // to get only a subset of the notes if desired.
        NoteRecord[] noterecs = api.getNotesForJournal(session_key, jid, "", "", "");
        for (NoteRecord noterec : noterecs) {
            out.format("... note: %s %s\n", noterec.getId(), noterec.getThe_text());
        }

        // If we know the id of a note we can modify the text.  The
        // first note we created is actually the second one in the
        // journal, because the journal has an automatically created
        // time stamp note at the beginning.
        api.modifyNoteText(session_key, noterecs[1].getId(), "modified note test one");

        // Once you modify a note, the ids of the underlying records may
        // change, so you must retrieve the note records again if you
        // want to continue to work with them.
        noterecs = api.getNotesForJournal(session_key, jid, "", "", "");

        // Now clean up after ourselves by deleting the notes.  Keep in
        // mind that we cannot delete the initial (timestamp) note of
        // the journal, other than by deleting the journal itself.  We
        // can test this by comparing the id of the note to the starting
        // id of the journal.
        for (NoteRecord noterec : noterecs) {
            if (!noterec.getId().equals(noterec.getNote_rec_start())) {
                api.deleteNote(session_key, noterec.getId());
            }
        }

        // Notes can also be attached to specific entries, in addition
        // to being part of a particular journal.  Let's try this with a
        // time entry.
        TimeRecord trec = api.getDefaultTimeRecord(session_key);
        trec.setHours(2.0);
        trec.setDate("20110801");
        trec.setProject(project_id);
        String trec_id = api.addFullTimeRecord(session_key, trec);

        api.createNote(session_key, jid, "test time record note", "time", trec_id);

        // Now we can fetch the note by the id of the entry it is
        // attached to.  If there are multiple notes attached to this
        // entry, their text will be concatenated, separated by the
        // "separator" argument.
        String note = api.getEntryNoteById(session_key, "time", trec_id, "");
        out.format("time entry note: %s\n", note);

        // Clean up.
        api.removeTimeRecord(session_key, trec_id);
        api.deleteJournal(session_key, jid);
        api.removeProject(session_key, project_id);
    }

    /*
    public void demoRateRules() throws RemoteException {
        // First, create and modify an individual RateRule.

        RateRule rule = (RateRule) app.newRecord(RateRule.class);
        rule.setId("");
        rule.setName("testrule0");
        rule.setDescription("no description yet");
        rule.setProject(soapStringArray(new String[] {"wildProj"}));
        rule.setCode(soapStringArray(new String[] {"wildCode"}));
        rule.setSubcode(soapStringArray(new String[] {"wildSub"}));
        rule.setOthercode(soapStringArray(new String[] {"wildOther"}));
        rule.setHours(40.0);
        rule.setTimeframe("Week");
        rule.setCondition("g");
        rule.setDays(soapStringArray(new String[0]));

        String rule_id = api.createRateRule(session_key, rule);
        out.format("rule created: %s\n", rule_id);

        // Read and modify it.
        rule = api.getRateRule(session_key, "testrule0");
        rule.setDescription("40 hour work week");
        String rule_id2 = api.modifyRateRule(session_key, "testrule0", rule);
        assert rule_id.equals(rule_id2);

        // Return the names of all rules.
        String[] rulenames = api.getRateRuleNames(session_key).getString();
        for (int i=0; i<rulenames.length; i++) {
            out.format("... rule: %s\n", rulenames[i]);
        }

        // Now create a rule set encompassing the rule we just created.
        RuleSet ruleset = new RuleSet();
        ruleset.setName("testruleset0");
        ruleset.setDescription("none yet");
        ruleset.setPayRateOrigin("User");
        ruleset.setBillRateOrigin("-none-");

        RuleSetRow row0 = (RuleSetRow) app.newRecord(RuleSetRow.class);
        row0.setRule("testrule0");
        row0.setPaytype("Multiply");
        row0.setPayamount(1.5);

        RuleSetRow[] rows = new RuleSetRow[] { row0 };
        Array_of_RuleSetRow soap_rows = new Array_of_RuleSetRow();
        soap_rows.setRuleSetRow(rows);
        ruleset.setRules(soap_rows);

        String ruleset_id = api.createRuleSet(session_key, ruleset);

        // We can modify it and get names just like with rules.
        ruleset = api.getRuleSet(session_key, "testruleset0");
        ruleset.setDescription("time and a half");
        String ruleset_id2 = api.modifyRuleSet(session_key, "testruleset0", ruleset);
        assert ruleset_id.equals(ruleset_id2);

        String[] rulesetnames = api.getRuleSetNames(session_key).getString();
        for (int i=0; i<rulesetnames.length; i++) {
            out.format("... ruleset: %s\n", rulesetnames[i]);
        }

        // Now show how to assign or remove it from a user.
        api.assignRuleSetToUser(session_key, app.testuser_id, "testruleset0");
        api.assignRuleSetToUser(session_key, app.testuser_id, "");

        // And finally clean up.
        api.deleteRuleSet(session_key, "testruleset0");
        api.deleteRateRule(session_key, "testrule0");
    }

    */
    public void demoApprovalPlans() throws RemoteException {
        String thedate = "20110815";

        // Create an approval plan.
        ApprovalObject approval = (ApprovalObject) app.newRecord(ApprovalObject.class);
        approval.setName("Test Approval Plan");
        approval.setType("time");
        approval.setBizid("default2");
        approval.setUserbefore(1);
        approval.setUserafter(2);
        approval.setApproverafter(3);
        approval.setBackupafter(4);
        approval.setUserrejected(1);
        approval.setUserapproved(1);
        approval.setApproversubmitted(1);
        approval.setApprover_1(jxapi_demo.USER);
        approval.setApprover_2(jxapi_demo.USER);
        approval.setBackup_approver_1(jxapi_demo.testuser_id);

        api.createApprovalPlan(session_key, approval);
        
        // Retrieve them. Pass an empty list to get all of them.
        String[] strs = new String[0];

        ApprovalObject[] approvals = api.getApprovalPlans(session_key, strs);
        for (ApprovalObject approval1 : approvals) {
            out.format("... approval plan %s\n", approval1.getId());
        }

        // Assign it to the test user.
        strs = new String[] { jxapi_demo.testuser_id };
        api.assignApprovalPlan(session_key, approvals[0].getId(), strs);

        // To demonstrate submission and approval we need to be logged
        // in as the test user.
        String session2_key;
        try {
            session2_key = api.login(jxapi_demo.testuser_id, "test", 1);
        } catch (RemoteException e) {
            api.changeUserPassword(jxapi_demo.testuser_id, jxapi_demo.testuser_id, "test");
            session2_key = api.login(jxapi_demo.testuser_id, "test", 1);
        }

        // As the second user, begin by accessing and thus implicitly
        // creating the sheet for our test date.
        String sheet_id = api.getSheetIDByDate(session2_key, "time", thedate);

        // It doesn't have any hours, but for this demo it doesn't need
        // any... submit it.
        api.submitSheet(session2_key, "time", sheet_id);

        // Now back to the admin user, let's reject the sheet.
        SheetStatusRecord newstatus = new SheetStatusRecord();
        newstatus.setId(sheet_id);
        newstatus.setChange("reject");
        newstatus.setReason("test rejection");
        newstatus.setStatus("");

        SheetStatusRecord[] stati = new SheetStatusRecord[] { newstatus };
        SheetStatusRecord[] results = api.changeSheetStatus(session_key, stati);
        for (SheetStatusRecord result : results) {
            out.format("... sheet status %s, %s, %s, %s\n", result.getId(), result.getChange(),
                    result.getReason(), result.getStatus());
        }

        // Now unassign the approval plan.  (There is currently no way
        // to delete it from the jxAPI.)
        strs = new String[] {jxapi_demo.testuser_id};
        api.unassignApprovalPlan(session_key, approvals[0].getId(), strs);
    }
}
