package com.journyx.jxapidemo;

import jxapi.*;
import com.journyx.jxapi.*;

import java.rmi.RemoteException;
import static java.lang.System.out;

public class jxapi_attribute_demo {
    static jxapi_demo app;
    static String session_key;
    static JxapiStub api;

    public jxapi_attribute_demo(jxapi_demo demoapp) {
        app = demoapp;
        session_key = jxapi_demo.session_key;
        api = jxapi_demo.api;
    }

    public void run () throws RemoteException {
        demoAttributes();
    }

    public void demoAttributes() throws RemoteException {
        // Start by looking up an attribute we know is always there in
        // the default installation.
        String attr_id = api.getAttributeTypeByName(session_key, "projects", "Budget");

        // Rename it.
        api.modifyAttributeTypeName(session_key, attr_id, "Planned Expenditures");
        api.modifyAttributeTypeDescription(session_key, attr_id, "How much will we spend?");

        // Retrieve the full record.
        AttributeTypeRecord attr_rec = api.getAttributeTypeRecordByName(session_key, "projects", "Planned Expenditures");
        out.format("... attribute %s %s\n", attr_rec.getPname(), attr_rec.getAttr_type());

        // Now rename it back.
        api.modifyAttributeTypeName(session_key, attr_id, "Budget");

        // Now let's add a new a new attribute, checking first to see if
        // the data type is valid.  We'll use the root project to set a
        // value and make sure it works.

        assert api.checkAttributeDataType(session_key, "STRING") != 0;
        attr_id = api.addAttributeType(session_key, "projects", "jxapi demo 1", "STRING");
        api.setAttributeTypeDefaultValue(session_key, attr_id, "default value");
        assert api.getAttributeTypeDefaultValue(session_key, attr_id).equals("default value");
        api.removeAttributeTypeDefaultValue(session_key, attr_id);

        assert api.checkAttributeValue(session_key, "STRING", "test value") != 0;
        api.setAttribute(session_key, "projects", "root", attr_id, "test value", true);
        String attr_value = api.getAttribute(session_key, "projects", "root", attr_id);
        assert attr_value.equals("test value");

        api.deleteAttribute(session_key, "projects", "root", attr_id);
        api.deleteAttributeType(session_key, attr_id);

        // Because of limitations on overloading in the jxAPI, there are
        // parallel methods for getAttribute, setAttribute,
        // checkAttributeValue, and {get,set}AttributeTypeDefaultValue
        // which accept and/or return integer and floating point values.
        // The following code is exactly like that demonstrated above,
        // but tests integer values instead.

        assert api.checkAttributeDataType(session_key, "INTEGER") != 0;
        attr_id = api.addAttributeType(session_key, "projects", "jxapi demo 2", "INTEGER");
        api.setAttributeTypeDefaultValueInteger(session_key, attr_id, 42);
        assert api.getAttributeTypeDefaultValueInteger(session_key, attr_id) == 42;
        api.removeAttributeTypeDefaultValue(session_key, attr_id);

        assert api.checkAttributeValueInteger(session_key, "INTEGER", 23) != 0;
        api.setAttributeInteger(session_key, "projects", "root", attr_id, 23, true);
        int attr_value_int = api.getAttributeInteger(session_key, "projects", "root", attr_id);
        assert attr_value_int == 23;

        api.deleteAttribute(session_key, "projects", "root", attr_id);
        api.deleteAttributeType(session_key, attr_id);

        // And now for floating point.
        assert api.checkAttributeDataType(session_key, "NUMBER") != 0;
        attr_id = api.addAttributeType(session_key, "projects", "jxapi demo 3", "NUMBER");
        api.setAttributeTypeDefaultValueNumber(session_key, attr_id, 42.1);
        assert Math.abs(42.1-api.getAttributeTypeDefaultValueNumber(session_key, attr_id)) < 0.1;
        api.removeAttributeTypeDefaultValue(session_key, attr_id);

        assert api.checkAttributeValueNumber(session_key, "NUMBER", 23.1) != 0;
        api.setAttributeNumber(session_key, "projects", "root", attr_id, 23.1, true);
        double attr_value_num = api.getAttributeNumber(session_key, "projects", "root", attr_id);
        assert Math.abs(23-attr_value_num) < 0.1;

        api.deleteAttribute(session_key, "projects", "root", attr_id);
        api.deleteAttributeType(session_key, attr_id);


        // To demonstrate selection values we will now add another
        // attribute.
        attr_rec = (AttributeTypeRecord) app.newRecord(AttributeTypeRecord.class);
        attr_rec.setAttr_type("ENUM_STRING_20");
        attr_rec.setDescription("jxapi demo attribute");
        attr_rec.setPname("jxapi_demo_attribute");
        attr_rec.setTable_name("projects");

        attr_id = api.addFullAttributeType(session_key, attr_rec);

        // Add some selection values.  Note that for integer and number
        // attributes there are parallel functions
        // addAttributeTypeSelectionValue{Number,Integer}, not shown
        // here.
        String[] selections = new String[] { "one", "two", "3", "four", "55555"};
        for (String selection : selections) {
            api.addAttributeTypeSelectionValue(session_key, attr_id, selection, false);
        }

        String[] values = api.getAttributeTypeSelectionValues(session_key, attr_id);
        out.format("... selection values %s\n", app.formatList(values));

        // Add another value and make it the default using its id.
        String value_id = api.addAttributeTypeSelectionValue(session_key, attr_id, "foo", false);
        api.setAttributeTypeSelectionDefault(session_key, value_id);

        // Now let's create a project and see if it gets the default
        // value.
        ProjectRecord prec = api.getDefaultProject(session_key);
        prec.setParent("root");
        prec.setName("test project");
        prec.setId("testprojecta00");
        prec.setCreator(jxapi_demo.USER);
        String proj_id = api.addFullProject(session_key, prec);

        out.format("... attribute for new project: %s\n", 
                api.getAttribute(session_key, "projects", "testprojecta00", attr_id));

        // Delete an individual selection value.
        api.deleteAttributeTypeSelectionValue(session_key, value_id);

        // Clean up.
        api.removeProject(session_key, proj_id);
        api.deleteAttributeType(session_key, attr_id);
    }
}
